Package: blockr.dock
Title: A Docking Layout Manager for 'blockr'
Version: 0.1.1
Authors@R: 
    c(person(given = "Nicolas",
             family = "Bennett",
             role = c("aut", "cre"),
             email = "nicolas@cynkra.com"),
      person(given = "David",
             family = "Granjon",
             role = c("aut"),
             email = "david@cynkra.com")
    )
Description: Building on the docking layout manager provided by 'dockViewR',
    this provides a flexible front-end to 'blockr.core'. It provides an extension mechanism which allows for providing means to manipulate a board
    object via panel-based user interface components.
URL: https://bristolmyerssquibb.github.io/blockr.dock/
BugReports: https://github.com/BristolMyersSquibb/blockr.dock/issues
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: blockr.core (>= 0.1.2), bsicons, shiny, shinyjs, bslib, glue,
        dockViewR (>= 0.2.1), htmltools, cli, shinyWidgets, jsonlite
Suggests: testthat (>= 3.0.0), DT, colorspace, methods, withr,
        shinytest2, knitr, rmarkdown
Config/testthat/edition: 3
Collate: 'action-class.R' 'action-block.R' 'action-link.R'
        'action-modal.R' 'action-stack.R' 'action-ui.R'
        'action-utils.R' 'block-meta.R' 'block-ui.R' 'board-plugins.R'
        'board-server.R' 'board-ui.R' 'dock-board.R' 'dock-stack.R'
        'ext-class.R' 'ext-edit.R' 'ext-ui.R' 'layout-class.R'
        'plugin-block.R' 'plugin-serdes.R' 'utils-dock.R' 'utils-id.R'
        'utils-misc.R' 'utils-pkg.R' 'utils-serdes.R' 'utils-serve.R'
        'utils-ui.R' 'layouts-class.R' 'view-ui.R'
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-29 12:37:55 UTC; dev
Author: Nicolas Bennett [aut, cre],
  David Granjon [aut]
Maintainer: Nicolas Bennett <nicolas@cynkra.com>
Repository: CRAN
Date/Publication: 2026-04-29 13:10:17 UTC
Built: R 4.6.0; ; 2026-05-01 22:40:42 UTC; unix
