# Generated by roxygen2: do not edit by hand

S3method("[",dock_extensions)
S3method("active_view<-",dock_board)
S3method("active_view<-",dock_views)
S3method(action_id,"function")
S3method(action_id,action)
S3method(active_view,dock_board)
S3method(active_view,dock_views)
S3method(all.equal,dock_grid)
S3method(apply_board_update,dock_board)
S3method(as.character,panel_ref)
S3method(as.list,dock_extensions)
S3method(as.list,dock_grid)
S3method(as.list,dock_layout)
S3method(as.list,dock_stack)
S3method(as_block_handle_id,block_handle_id)
S3method(as_block_handle_id,block_panel_id)
S3method(as_block_handle_id,blocks)
S3method(as_block_handle_id,character)
S3method(as_block_handle_id,dock_id)
S3method(as_block_handle_id,list)
S3method(as_block_panel_id,block_handle_id)
S3method(as_block_panel_id,block_panel_id)
S3method(as_block_panel_id,blocks)
S3method(as_block_panel_id,character)
S3method(as_block_panel_id,dock_id)
S3method(as_block_panel_id,list)
S3method(as_dock_board,board)
S3method(as_dock_board,dock_board)
S3method(as_dock_extensions,dock_extension)
S3method(as_dock_extensions,dock_extensions)
S3method(as_dock_extensions,list)
S3method(as_dock_grid,dock_grid)
S3method(as_dock_grid,dock_layout)
S3method(as_dock_grid,list)
S3method(as_dock_handle_id,blocks)
S3method(as_dock_handle_id,board)
S3method(as_dock_handle_id,character)
S3method(as_dock_handle_id,dock_extension)
S3method(as_dock_handle_id,dock_extensions)
S3method(as_dock_handle_id,dock_grid)
S3method(as_dock_handle_id,dock_handle_id)
S3method(as_dock_layout,dock_grid)
S3method(as_dock_layout,dock_layout)
S3method(as_dock_layout,list)
S3method(as_dock_panel_id,blocks)
S3method(as_dock_panel_id,board)
S3method(as_dock_panel_id,character)
S3method(as_dock_panel_id,dock_extension)
S3method(as_dock_panel_id,dock_extensions)
S3method(as_dock_panel_id,dock_grid)
S3method(as_dock_panel_id,dock_panel_id)
S3method(as_dock_stack,dock_stack)
S3method(as_dock_stack,list)
S3method(as_dock_stack,stack)
S3method(as_dock_view,dock_layout)
S3method(as_dock_view,dock_view)
S3method(as_ext_handle_id,block_panel_id)
S3method(as_ext_handle_id,character)
S3method(as_ext_handle_id,dock_extension)
S3method(as_ext_handle_id,dock_extensions)
S3method(as_ext_handle_id,dock_id)
S3method(as_ext_handle_id,ext_handle_id)
S3method(as_ext_handle_id,ext_panel_id)
S3method(as_ext_handle_id,list)
S3method(as_ext_panel_id,character)
S3method(as_ext_panel_id,dock_extension)
S3method(as_ext_panel_id,dock_extensions)
S3method(as_ext_panel_id,dock_id)
S3method(as_ext_panel_id,ext_handle_id)
S3method(as_ext_panel_id,ext_panel_id)
S3method(as_ext_panel_id,list)
S3method(as_obj_id,block_handle_id)
S3method(as_obj_id,block_panel_id)
S3method(as_obj_id,character)
S3method(as_obj_id,dock_panel_id)
S3method(as_obj_id,ext_handle_id)
S3method(as_obj_id,ext_panel_id)
S3method(as_obj_id,list)
S3method(as_obj_id,view_handle_id)
S3method(as_view_handle_id,character)
S3method(as_view_handle_id,view_handle_id)
S3method(augment_board_update,dock_board)
S3method(block_ui,dock_board)
S3method(blockr.core::blockr_test_exports,dock_board)
S3method(blockr_app_options,dock_board)
S3method(blockr_app_server,dock_board)
S3method(blockr_app_ui,dock_board)
S3method(blockr_deser,dock_board)
S3method(blockr_deser,dock_extension)
S3method(blockr_deser,dock_extensions)
S3method(blockr_deser,dock_grids)
S3method(blockr_deser,dock_stack)
S3method(blockr_deser,dock_view)
S3method(blockr_deser,dock_views)
S3method(blockr_ser,dock_extension)
S3method(blockr_ser,dock_extensions)
S3method(blockr_ser,dock_grids)
S3method(blockr_ser,dock_view)
S3method(blockr_ser,dock_views)
S3method(board_actions,dock_board)
S3method(board_actions,dock_extension)
S3method(board_options,dock_extension)
S3method(board_plugins,dock_board)
S3method(board_ui,dock_board)
S3method(extension_block_callback,dock_extension)
S3method(external_ctrl_vars,dock_extension)
S3method(format,dock_grid)
S3method(format,dock_stack)
S3method(insert_block_ui,dock_board)
S3method(print,dock_grid)
S3method(print,panel_ref)
S3method(remove_block_ui,dock_board)
S3method(rm_blocks,dock_board)
S3method(serialize_board,dock_board)
S3method(stack_color,board)
S3method(stack_color,dock_stack)
S3method(stack_color,stack)
S3method(stack_color,stacks)
S3method(str,dock_extension)
S3method(str,dock_extensions)
S3method(str,dock_grid)
S3method(str,dock_view)
S3method(str,dock_views)
S3method(str_value,dock_board)
S3method(str_value,dock_extension)
S3method(str_value,dock_extensions)
S3method(str_value,dock_grid)
S3method(str_value,dock_stack)
S3method(str_value,dock_view)
S3method(str_value,dock_views)
S3method(update_stack,dock_stack)
S3method(validate_board,dock_board)
S3method(validate_board_update,dock_board)
S3method(validate_extension,dock_extension)
S3method(validate_stack,dock_stack)
export("active_view<-")
export("board_grids<-")
export("board_views<-")
export("dock_extensions<-")
export("stack_color<-")
export("view_name<-")
export(action_id)
export(action_triggers)
export(active_view)
export(as_block_handle_id)
export(as_block_panel_id)
export(as_dock_board)
export(as_dock_extensions)
export(as_dock_grid)
export(as_dock_handle_id)
export(as_dock_layout)
export(as_dock_panel_id)
export(as_dock_stack)
export(as_dock_view)
export(as_ext_handle_id)
export(as_ext_panel_id)
export(as_obj_id)
export(as_view_handle_id)
export(blk)
export(blk_color)
export(blk_icon_data_uri)
export(blks_metadata)
export(block_input_select)
export(block_registry_selectize)
export(block_status_badge)
export(board_actions)
export(board_grids)
export(board_select)
export(board_views)
export(default_layout)
export(dock_board_options)
export(dock_ext_ids)
export(dock_extensions)
export(dock_grid)
export(dock_id)
export(dock_view)
export(ext)
export(extension_block_callback)
export(extension_ctor)
export(extension_description)
export(extension_id)
export(extension_ids)
export(extension_name)
export(extension_server)
export(extension_ui)
export(group)
export(is_action)
export(is_action_generator)
export(is_dock_board)
export(is_dock_extension)
export(is_dock_extensions)
export(is_dock_grid)
export(is_dock_grids)
export(is_dock_layout)
export(is_dock_stack)
export(is_dock_view)
export(is_dock_views)
export(is_panel_ref)
export(layout_panel_ids)
export(new_action)
export(new_dock_board)
export(new_dock_extension)
export(new_dock_extensions)
export(new_dock_layout)
export(new_dock_stack)
export(new_edit_board_extension)
export(panel_obj_ids)
export(panels)
export(stack_color)
export(suggest_new_colors)
export(validate_dock_grid)
export(validate_dock_grids)
export(validate_dock_layout)
export(validate_dock_view)
export(validate_dock_views)
export(validate_extension)
export(validate_extensions)
export(view_members)
export(view_name)
export(view_names)
import(blockr.core)
import(bslib)
import(shiny)
importFrom(glue,glue)
importFrom(utils,str)
